/*************************************************************************
 * File Name     : EDIT.C
 *
 * Description   : This sample program demonstrates how to use the API
 *                 LayoutEditShape. This API is an
 *                 Arabic specific one, and it is used to shape
 *                 (according to the value of the TextShaping value in the
 *                 LayoutTextDescriptor) one character knowing its surronding
 *                 characters (which may get changed as well)
 *
 * Concepts      : When using this API in the EditReplace mode, it will
 *                 read the current charater as defined by index, and any
 *                 surrounding (front and/or back) as defined by the
 *                 ShapeContextSize layout value.
 *                 Based on these readings, the shape of the current
 *                 character is determined, also the surrounding characters
 *                 may change. The output is written to OutBuf.
 *                 On return, the index argument is modified to indicate the
 *                 offset to the first code element in InpBuf that required
 *                 shaping.
 *                 If the set of surrounding characters as defined by index,
 *                 Inpbuf and InpSize is less than the size of front/back of
 *                 ShapeContextSize, the API will assume there is no additional
 *                 context available.
 *                 On return, the OutSize argument is modified to the actual
 *                 number of bytes placed in OutBuf.
 *
 *                 When using this API in the EditInput mode, it will
 *                 read the current character as defined by index and any
 *                 preceding(front) elements as defined by the ShapeContextSize.
 *
 *                 When ActiveShape Editing is set, the LayoutObject maintains an
 *                 EditInput state, that may affect subsequent calls to the
 *                 API.
 *
 * API's         : LayoutTransformText
 *                 LayoutQueryValues
 *                 LayoutSetValues
 *                 VioWrtCharStr
 *                 VioScrollUp
 *                 VioSetCurType
 *
 * Required files: To build and run this sample code the following files
 *                 are needed:
 *                 - EDIT.C
 *                 - EDIT.DEF
 *                 - EDIT.MAK
 *                 - EDIT.LNK
 *
 *  Copyright (C) 1991 IBM Corporation
 *
 *      DISCLAIMER OF WARRANTIES.  The following [enclosed] code is
 *      sample code created by IBM Corporation. This sample code is not
 *      part of any standard or IBM product and is provided to you solely
 *      for  the purpose of assisting you in the development of your
 *      applications.  The code is provided "AS IS", without
 *      warranty of any kind.  IBM shall not be liable for any damages
 *      arising out of your use of the sample code, even if they have been
 *      advised of the possibility of such damages.                                                    *
 ************************************************************************/
#define INCL_BASE
#define INCL_DOSERRORS

#include <string.h>
#include <os2.h>
#include <conio.h>

/* To include the neccessary header files for Layout Transform */
#include <layout.h>


VOID Initialize(VOID);
VOID ShapeAndDisplay(VOID);
VOID SetCur(INT);



/* Define the parameter to Hide and Restore the cursor */
#define HIDE   -1
#define RESTORE 0

/* Define the buffer for the string to be displayed */

CHAR cString[50];

/* Defines for the layout-object_transform  */
LAYOUT_OBJECT          hlo;
LAYOUT_VALUES          layout[3];
LAYOUT_TEXT_DESCRIPTOR  descr;
PLAYOUT_TEXT_DESCRIPTOR Pdescr;


/*************************************************************************
 * Name          : Main
 *
 * Description   : This is the main part of the program, it first clears
 *                 the screen and then hide the cursor. The character in
 *                 the string to be shaped will be displayed before and
 *                 LayoutTransformText is called.
 *
 * API's         : NlsSetBidiAtt
 *                 LayoutCreateObject
 *                 LayoutDestroyObject
 *                 VioScrollUp
 *
 * Parameters    : This program does not take any command line parameters
 *
 * Returns       : None
 *
 ************************************************************************/

INT main(SHORT sArgc,CHAR **ppArgv)
{
   APIRET   RC, Rc;
   CHAR     msg[50];
   ULONG    len;

   /* clear the screen  */
   VioScrollUp(0, 0, 24, 79, -1," \x07", 0);

   /* hide the cursor */
   SetCur(HIDE);

   /* Create the Layout Object */

   RC=LayoutCreateObject(Locale_Arabic,&hlo);

   if (RC)
   {
   /* Get a message from the message file EDIT.msg */
     Rc = DosGetMessage(0,0, msg, sizeof(msg),
                        0,"EDIT.msg",&len);
     if (Rc)
      {
        printf("DosGetMessage Error RC=%d\n",Rc);
        exit(1);
       }
    msg[len]=0;
    printf("\n\n%s %d\n",msg, RC);
    exit(1);
    }


   /* Write the static text to the screen and the string that
    * needs to be editshaped */
   Initialize();

   /* Call the API to shape the string and write it to the screen */
   ShapeAndDisplay();

   /* Wait for Enter key to end the program */

    Rc = DosGetMessage(0,0, msg, sizeof(msg),
                       9,"EDIT.msg",&len);
    if (Rc)
     {
       printf("DosGetMessage Error RC=%d\n",Rc);
       exit(1);
      }
   msg[len]=0;
   VioWrtCharStr(msg, 32, 23, 23, 0l);
   getchar();

   /* clear the screen before exit */
   VioScrollUp(0, 0, 24, 79, -1," \x07", 0);

   /* restore the cursor position */
   SetCur(RESTORE);

   /* Free the Layout Object */
   LayoutDestroyObject(hlo);


   return (0);
}
/*************************************************************************
 * Name          : INITIALIZE
 *
 * Description   : This function saves the current Bidi attributes of
 *                 the session in a global variable, and sets session with
 *                 its own Bidi attributes to protect the displayed shaped
 *                 string from been processed again according to the session
 *                 Bidi attributes.
 *                 Then the function displays some static text and also the
 *                 String to be shaped before any shaping processing.
 *
 * API's         : NlsQueryBidiAtt
 *                 NlsSetBidiAtt
 *                 VioWrtCharStr
 *
 * Parameters    : This program does not take any command line parameters
 *
 * Returns       : None
 *
 ************************************************************************/
VOID Initialize(VOID)
{
   INT iI, iRow, iCol;

   APIRET  RC,Rc;
   CHAR    msg[50];
   ULONG   len;

   /* define structures to be array of strings to contain the static
    * text to be wirtten on the screen  */
   struct {
          CHAR cScreen[42];
          }mssg1[5];

   struct {
           CHAR cScreen[15];
           }mssg2[3];


   /* Initialize the structures with the characters to be displayed */
   Rc = DosGetMessage(0,0, mssg1[0].cScreen, sizeof(mssg1[0].cScreen),
                      3,"EDIT.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg1[0].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg1[1].cScreen, sizeof(mssg1[1].cScreen),
                      4,"EDIT.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg1[1].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg1[2].cScreen, sizeof(mssg1[2].cScreen),
                      5,"EDIT.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg1[2].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg1[3].cScreen, sizeof(mssg1[3].cScreen),
                      6,"EDIT.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg1[3].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg1[4].cScreen, sizeof(mssg1[4].cScreen),
                      7,"EDIT.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg1[4].cScreen[len]=0;

   /* In this for loop, we increment the iRow after the display of each
    * string in the structure to jump to another line, keeping the value
    * of the iCol value fixed to have the strings indented */
   for(iI = 0, iRow = 2, iCol = 20; iI < 4; iI++, iRow=iRow+5)
      VioWrtCharStr(mssg1[iI].cScreen, strlen(mssg1[iI].cScreen),
                    iRow, iCol, 0l);
   /* write the last message */
   VioWrtCharStr(mssg1[4].cScreen, strlen(mssg1[4].cScreen),
                 iRow-4, iCol, 0l);


   /* Initialize the second structure with the characters to be displayed */

   Rc = DosGetMessage(0,0, mssg2[0].cScreen, sizeof(mssg2[0].cScreen),
                      8,"EDIT.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg2[0].cScreen[len]=0;

   Rc = DosGetMessage(0,0, mssg2[1].cScreen, sizeof(mssg2[1].cScreen),
                      10,"EDIT.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   mssg2[1].cScreen[len]=0;

   /* This is the string that is going to be passed to LayoutEditShape */

   Rc = DosGetMessage(0,0, cString, sizeof(cString),
                      11,"EDIT.msg",&len);
   if (Rc)
    {
      printf("DosGetMessage Error RC=%d\n",Rc);
      exit(1);
     }
   cString[len]=0;


   for(iI = 0, iRow = 10, iCol = 32; iI <= 1; iI++, iRow = iRow+5)

   VioWrtCharStr(mssg2[iI].cScreen, strlen(mssg2[iI].cScreen), iRow, iCol, 0l);

}
/*************************************************************************
 * Name          : SHAPEANDDISPLAY
 *
 * Description   : This function will show how to use the Arabic API
 *                 LayoutEditShape , and how to fill the needed
 *                 structures.
 *                 The function will display the string after it is shaped.
 *
 * API's         : LayoutEditShape
 *                 VioWrtCharStr
 *
 * Parameters    : This program does not take any command line parameters
 *
 * Returns       : None
 *
 ************************************************************************/
VOID ShapeAndDisplay(VOID)
{
  APIRET RC,Rc;
  int    j;
  /* Define the output string*/
  UCHAR    cTarget[50]="";
  UCHAR    BidiLvl[50]="";
  ULONG    ToOutBuf[50];
  ULONG    ToInpBuf[50];
  ULONG    InSize, OutSize;
  ULONG    index;
  CHAR     msg[50];
  ULONG    len;

   /* Define the LayoutValues that will need changing */
   layout[0].name= TypeOfText|Orientation;
   layout[0].value=&descr;

   /* using the OR operator, we set the bits in the Layout Descriptor
    * to describe the orientation of the buffer and the Type of Text
    * that we want for the output */

   descr.in=TEXT_VISUAL|ORIENTATION_LTR;
   descr.out=TEXT_VISUAL|ORIENTATION_LTR;
   layout[1].name=0;           /* End of change */

   /* Set the LayoutValues   */
     RC=LayoutSetValues(hlo,layout,&index);

     if (RC)
     {
           Rc = DosGetMessage(0,0, msg, sizeof(msg),
                              12,"EDIT.msg",&len);
           if (Rc)
            {
              printf("DosGetMessage Error RC=%d\n",Rc);
              exit(1);
             }
           msg[len]=0;
           printf("%s %ld\n",msg,index);
           exit(0);
      }


  /* Here we point to the character were we want to shape, in
   * our case is the character that is newly introduced in
   * the string .. i.e the blank.  */
  index = 4L;

  /* The length of the whole buffer */
  InSize = strlen(cString);

  /* Call the API to shape the changed characters */
     RC=LayoutEditShape (hlo,EDITREPLACE,&index,
                                  (PUCHAR)cString, &InSize,
                                  (PUCHAR)cTarget, &OutSize);


  if(RC != 0)
  {
    Rc = DosGetMessage(0,0, msg, sizeof(msg),
                       13,"EDIT.msg",&len);
    if (Rc)
     {
       printf("DosGetMessage Error RC=%d\n",Rc);
       exit(1);
      }
    msg[len]=0;
    printf("%s %ld\n",msg,RC);
    exit(1);
  }
     /* Copy modified characters in cTarget into the input buffer
      * cString
      * InSize = Actual chars shaped in output buffer cTarget
      * OutSize = Size of Outbuf
      * index = position in input buffer of start of shaped       */

     for (j=0; j<InSize; j++) {
        cString[index+j] = cTarget[j];
     } /* endfor */


    VioWrtCharStr(cString, strlen(cString), 21, 32, 0l);
}

/*************************************************************************
 * Name          : SET_CUR
 *
 * Description   : This function hides or restores the cursor position.
 *
 * API's         : VioSetCurType
 *
 * Parameters    : (INT flag)
 *                 flag indicates whether we want ot hide or restore
 *                 the cursor.
 *
 * Returns       : None
 *
 ************************************************************************/

VOID SetCur(INT iFlag)
{
   HVIO HVIOhandle = 0;
   VIOCURSORINFO cursor;

   cursor.yStart  = 3;
   cursor.cEnd    = 5;
   cursor.cx      = 0;
   cursor.attr    = iFlag;

   VioSetCurType(&cursor, HVIOhandle);
}

